///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hbin;

import 'HVal.dart';
import 'str-utils.dart';

/// HBin models a binary file with a MIME type.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HBin extends HVal {

  /** Construct for MIME type */
  HBin(this.mime) {
    if (mime == null || mime.length == 0 || mime.indexOf('/') < 0) {
      throw new FormatException("Invalid mime val: \"$mime\"", mime);
    }
  }

  /** MIME type for binary file */
  final String mime;

  /** Hash code is based on mime field */
  int get hashCode => mime.hashCode;

  /** Equals is based on mime field */
  bool operator ==(Object that) => that is HBin && this.mime == (that as HBin).mime;

  /** Encode as "Bin(<mime>)" */
  String _toZinc;
  String toZinc() {
    if (_toZinc == null) {
      StringBuffer s = new StringBuffer();
      s.write("Bin(");

      mime.codeUnits.forEach((int c) {
        if (c > 127 || c == RIGHT_CURLY_CHAR) throw new FormatException("Invalid mime, char='${new String.fromCharCode(c)}'", mime);
        s.writeCharCode(c);
      });

      s.write(')');
      _toZinc =  s.toString();
    }

    return _toZinc;
  }
}