///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

part of haystack.hgrid;

/// HRow is a row in a HGrid.  It implements the HDict interface also.
///
/// See [Project Haystack](http://project-haystack.org/doc/Grids)
class HRow extends HDict {

  HRow(this._grid, this._cells) : super._();

  /** Get the grid associated with this row */
  HGrid get grid => _grid;

  /** Number of columns in grid (which may map to null cells) */
  int get size => _grid.cols.length;

  /**
   * Get a cell by column name.  If the column is undefined or
   * the cell is null then raise UnknownNameException or return
   * null based on checked flag.
   */
  HVal get(String name, {bool checked: true}) {
    HCol col = _grid.colByName(name, false);
    if (col != null) {
      HVal val = _cells[col.index];
      if (val != null) return val;
    }

    if (checked) throw new UnknownNameError(name);
    return null;
  }

  /**
   * Get a cell by column.  If cell is null then raise
   * UnknownNameException or return  null based on checked flag.
   */
  HVal getByCol(HCol col, {bool checked: true}) {
    HVal val = _cells[col.index];
    if (val != null) return val;
    if (checked) throw new UnknownNameError(col.name);
    return null;
  }

  /**
   * Return Map.Entry name/value iterator which only includes
   * non-null cells
   */
  Iterator<DictEntry> iterator() => new _RowIterator(_grid, _cells);


//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  HGrid _grid;
  List<HVal> _cells;
}

class _RowIterator implements Iterator {

  _RowIterator(this._grid, this._cells);

  int _colId = -1;
  DictEntry _current;

  final HGrid _grid;
  final List<HVal> _cells;

  bool moveNext() {
    for (_colId++; _colId < _grid.cols.length; ++_colId) if (_cells[_colId] != null) break;
    if (_colId >= _grid.cols.length) return false;

    HVal val = _cells[_colId];
    String name = _grid.colByIndex(_colId).name;
    _current = new DictEntry(name, val);
    return true;
  }

  DictEntry get current => _current;

}